"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const electron_1 = require("electron");
const windowState = require("electron-window-state");
const menubar = require("menubar");
const config_1 = require("./config");
const account_switcher_1 = require("./account_switcher");
const log_1 = require("./log");
const common_1 = require("./common");
const ELECTRON_ISSUE_9230 = common_1.IS_WINDOWS || common_1.IS_LINUX;
function shouldOpenInternal(host, url) {
    if (host.startsWith('https://pawoo.net') && url.startsWith('https://accounts.pixiv.net/login?')) {
        log_1.default.debug('accounts.pixiv.net opens and will redirect to pawoo.net for signin with Pixiv account.');
        return true;
    }
    return false;
}
class Window {
    constructor(browser, state, account, menubar) {
        this.browser = browser;
        this.state = state;
        this.account = account;
        this.menubar = menubar;
        if (!common_1.IS_DARWIN) {
            browser.setMenu(electron_1.Menu.getApplicationMenu());
        }
        browser.webContents.on('will-navigate', (e, url) => {
            const host = config_1.hostUrl(this.account);
            if (url.startsWith(host)) {
                return;
            }
            if (shouldOpenInternal(host, url)) {
                return;
            }
            e.preventDefault();
            electron_1.shell.openExternal(url);
            log_1.default.debug('Opened URL with external browser (will-navigate)', url);
        });
        browser.webContents.on('new-window', (e, url) => {
            if (ELECTRON_ISSUE_9230) {
                return;
            }
            e.preventDefault();
            electron_1.shell.openExternal(url);
            log_1.default.debug('Opened URL with external browser (new-window)', url);
        });
        browser.webContents.session.setPermissionRequestHandler((contents, permission, callback) => {
            const url = contents.getURL();
            const grantedByDefault = url.startsWith(config_1.hostUrl(this.account)) &&
                permission !== 'geolocation' &&
                permission !== 'media';
            if (grantedByDefault) {
                log_1.default.debug('Permission was granted', permission);
                callback(true);
                return;
            }
            log_1.default.debug('Create dialog for user permission', permission);
            electron_1.dialog.showMessageBox({
                type: 'question',
                buttons: ['Accept', 'Reject'],
                message: `Permission '${permission}' is requested by ${url}`,
                detail: "Please choose one of 'Accept' or 'Reject'",
            }, (buttonIndex) => {
                const granted = buttonIndex === 0;
                callback(granted);
            });
        });
    }
    static create(account, config, mb = null) {
        return (config.normal_window ? startNormalWindow(account, config) : startMenuBar(account, config, mb))
            .then(win => {
            win.browser.webContents.on('dom-ready', () => {
                applyUserCss(win.browser, config);
                win.browser.webContents.setZoomFactor(config.zoom_factor);
                log_1.default.debug('Send config to renderer procress');
                win.browser.webContents.send('mstdn:config', config, account);
            });
            return win;
        });
    }
    open(url) {
        log_1.default.debug('Open URL:', url);
        this.browser.webContents.once('did-get-redirect-request', (e, _, newUrl) => {
            if (url === newUrl) {
                return;
            }
            log_1.default.debug('Redirecting to ' + newUrl + '. Will navigate to login page for user using single user mode');
            e.preventDefault();
            this.browser.loadURL(config_1.hostUrl(this.account) + '/auth/sign_in');
        });
        if (url.startsWith('https://pawoo.net')) {
            this.browser.loadURL(url, {
                userAgent: common_1.IOS_SAFARI_USERAGENT,
            });
        }
        else {
            this.browser.loadURL(url);
        }
    }
    close() {
        log_1.default.debug('Closing window:', this.account);
        this.state.saveState();
        this.state.unmanage();
        this.browser.webContents.removeAllListeners();
        this.browser.removeAllListeners();
        if (this.menubar) {
            delete this.menubar.window;
        }
        this.browser.close();
    }
    toggle() {
        if (this.menubar) {
            if (this.browser.isFocused()) {
                log_1.default.debug('Toggle window: shown -> hidden');
                this.menubar.hideWindow();
            }
            else {
                log_1.default.debug('Toggle window: hidden -> shown');
                this.menubar.showWindow();
            }
        }
        else {
            if (this.browser.isFocused()) {
                log_1.default.debug('Toggle window: shown -> hidden');
                if (common_1.IS_DARWIN) {
                    electron_1.app.hide();
                }
                else if (common_1.IS_WINDOWS) {
                    this.browser.minimize();
                }
                else {
                    this.browser.hide();
                }
            }
            else {
                log_1.default.debug('Toggle window: hidden -> shown');
                this.browser.show();
                if (common_1.IS_WINDOWS) {
                    this.browser.focus();
                }
            }
        }
    }
}
exports.default = Window;
function applyUserCss(win, config) {
    if (config.chromium_sandbox) {
        log_1.default.debug('User CSS is disabled because Chromium sandbox is enabled');
        return;
    }
    fs.readFile(common_1.USER_CSS, 'utf8', (err, css) => {
        if (err) {
            log_1.default.debug('Failed to load user.css: ', err.message);
            return;
        }
        win.webContents.insertCSS(css);
        log_1.default.debug('Applied user CSS:', common_1.USER_CSS);
    });
}
function startNormalWindow(account, config) {
    log_1.default.debug('Setup a normal window');
    return new Promise(resolve => {
        const state = windowState({
            defaultWidth: 600,
            defaultHeight: 800,
        });
        const win = new electron_1.BrowserWindow({
            width: state.width,
            height: state.height,
            x: state.x,
            y: state.y,
            icon: common_1.APP_ICON,
            show: false,
            autoHideMenuBar: !!config.hide_menu,
            webPreferences: {
                nodeIntegration: false,
                sandbox: sandboxFlag(config, account),
                preload: common_1.PRELOAD_JS,
                partition: account_switcher_1.partitionForAccount(account),
                zoomFactor: config.zoom_factor,
            },
        });
        win.once('ready-to-show', () => {
            win.show();
        });
        win.once('closed', () => {
            electron_1.app.quit();
        });
        if (state.isFullScreen) {
            win.setFullScreen(true);
        }
        else if (state.isMaximized) {
            win.maximize();
        }
        state.manage(win);
        win.webContents.once('dom-ready', () => {
            log_1.default.debug('Normal window application was launched');
            if (common_1.IS_DEBUG) {
                win.webContents.openDevTools({ mode: 'detach' });
            }
        });
        resolve(new Window(win, state, account, null));
    });
}
function sandboxFlag(config, account) {
    const electronBugIssue9276 = account.host === 'mstdn.jp' || account.host === 'https://mstdn.jp';
    if (electronBugIssue9276) {
        return false;
    }
    return !!config.chromium_sandbox;
}
function startMenuBar(account, config, bar) {
    log_1.default.debug('Setup a menubar window');
    return new Promise(resolve => {
        const state = windowState({
            defaultWidth: 320,
            defaultHeight: 420,
        });
        const icon = common_1.trayIcon(config.icon_color);
        const mb = bar || menubar({
            icon,
            width: state.width,
            height: state.height,
            alwaysOnTop: common_1.IS_DEBUG || !!config.always_on_top,
            tooltip: 'Mstdn',
            autoHideMenuBar: !!config.hide_menu,
            show: false,
            showDockIcon: true,
            webPreferences: {
                nodeIntegration: false,
                sandbox: sandboxFlag(config, account),
                preload: common_1.PRELOAD_JS,
                partition: account_switcher_1.partitionForAccount(account),
                zoomFactor: config.zoom_factor,
            },
        });
        mb.once('after-create-window', () => {
            log_1.default.debug('Menubar application was launched');
            if (common_1.IS_DEBUG) {
                mb.window.webContents.openDevTools({ mode: 'detach' });
            }
            state.manage(mb.window);
            resolve(new Window(mb.window, state, account, mb));
        });
        mb.once('after-close', () => {
            electron_1.app.quit();
        });
        if (bar) {
            log_1.default.debug('Recreate menubar window with different partition:', account);
            const pref = mb.getOption('webPreferences');
            pref.partition = account_switcher_1.partitionForAccount(account);
            pref.sandbox = sandboxFlag(config, account);
            mb.setOption('webPreferences', pref);
            mb.showWindow();
        }
        else {
            log_1.default.debug('New menubar instance was created:', account);
            mb.once('ready', () => mb.showWindow());
        }
    });
}
//# sourceMappingURL=window.js.map