"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const electron_1 = require("electron");
const Mousetrap = require("mousetrap");
const Ipc = require("./ipc");
const log_1 = require("./log");
function scrollable() {
    const scrollable = document.querySelector('.scrollable');
    if (!scrollable) {
        log_1.default.error('Scrollable element was not found!');
        return { scrollTop: 0 };
    }
    return scrollable;
}
function navigateTo(host, path) {
    const url = `https://${host}${path}`;
    if (window.location.href === url) {
        log_1.default.info('Current URL is already', url);
        return;
    }
    const link = document.querySelector(`a[href="${path}"]`);
    if (link) {
        log_1.default.info('Click link by shortcut', path);
        link.click();
    }
    else {
        log_1.default.info('Force navigation by shortcut', path);
        window.location.href = url;
    }
}
const ShortcutActions = {
    'scroll-top': () => {
        scrollable().scrollTop = 0;
    },
    'scroll-bottom': () => {
        scrollable().scrollTop = document.body.scrollHeight;
    },
    'scroll-down': () => {
        scrollable().scrollTop += window.innerHeight / 3;
    },
    'scroll-up': () => {
        scrollable().scrollTop -= window.innerHeight / 3;
    },
    'next-account': () => {
        Ipc.send('mstdn:next-account');
    },
    'prev-account': () => {
        Ipc.send('mstdn:prev-account');
    },
    'open-in-browser': () => {
        electron_1.remote.shell.openExternal(window.location.href);
    }
};
function setupKeymaps(config, account, loader) {
    const dataDir = config.__DATA_DIR || '/';
    for (const key in config.keymaps) {
        const action = config.keymaps[key];
        if (action.endsWith('.js')) {
            if (config.chromium_sandbox) {
                log_1.default.info('Loading external script is limited because Chromium sandbox is enabled. Disable shortcut:', action);
                continue;
            }
            const script = path.join(dataDir, action);
            let plugin;
            try {
                plugin = require(script);
            }
            catch (e) {
                log_1.default.error('Error while loading plugin ' + script, e);
                continue;
            }
            Mousetrap.bind(key, e => {
                e.preventDefault();
                log_1.default.info('Shortcut:', action);
                try {
                    plugin(config, account);
                }
                catch (e) {
                    log_1.default.error('Failed to run shortcut script ' + script, e);
                }
            });
        }
        else if (action.startsWith('plugin:')) {
            const split = action.split(':').slice(1);
            if (split.length <= 1) {
                log_1.default.error('Invalid format keymap. Plugin-defined action should be "plugin:{name}:{action}":', action);
                continue;
            }
            Mousetrap.bind(key, e => {
                loader.runKeyShortcut(e, split[0], split[1]);
            });
        }
        else if (action.startsWith('/')) {
            Mousetrap.bind(key, e => {
                e.preventDefault();
                navigateTo(account.host, action);
            });
        }
        else {
            const func = ShortcutActions[action];
            if (!func) {
                log_1.default.error('Unknown shortcut action:', action);
                continue;
            }
            Mousetrap.bind(key, e => {
                log_1.default.info('Shortcut:', action);
                e.preventDefault();
                func();
            });
        }
    }
}
exports.default = setupKeymaps;
//# sourceMappingURL=key_handler.js.map