"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const log_1 = require("./log");
class PluginsLoader {
    constructor(config, account) {
        this.config = config;
        this.account = account;
        this.tryLoading = () => {
            if (document.querySelector('[data-react-class="Mastodon"]') === null) {
                log_1.default.info('Root element of react app was not found. App seems not to be loaded yet.');
                return false;
            }
            for (const key in this.preloads) {
                const f = this.preloads[key].preload;
                if (!f) {
                    log_1.default.info('Plugin does not have preload function. Skipped:', key);
                    continue;
                }
                try {
                    f(this.config, this.account);
                }
                catch (e) {
                    log_1.default.error(`Error while loading plugin '${key}':`, e);
                }
            }
            log_1.default.info('Plugins were loaded:', this.preloads);
            return true;
        };
        this.loaded = false;
        this.preloads = {};
        if (config.chromium_sandbox) {
            log_1.default.info('Chromium sandbox is enabled. Plugin is disabled.');
            return;
        }
        const dir_base = path.join(config.__DATA_DIR, 'node_modules');
        for (const plugin of this.account.plugins || []) {
            const plugin_path = path.join(dir_base, `mstdn-plugin-${plugin}`);
            try {
                this.preloads[plugin_path] = require(plugin_path);
            }
            catch (e) {
                log_1.default.error(`Failed to load plugin ${plugin_path}:`, e);
            }
        }
    }
    loadAfterAppPrepared() {
        if (Object.keys(this.preloads).length === 0) {
            log_1.default.info('No Plugin found. Skip loading');
            this.loaded = true;
            return;
        }
        return new Promise(resolve => {
            window.requestIdleCallback(() => {
                log_1.default.debug('Start loading plugins', this.preloads);
                if (this.tryLoading()) {
                    return resolve();
                }
                this.observeAppPrepared(this.tryLoading).then(resolve);
            });
        });
    }
    observeAppPrepared(callback) {
        return Promise.resolve(callback());
    }
    findPluginByName(name) {
        const pluginName = `mstdn-plugin-${name}`;
        for (const p in this.preloads) {
            if (p.endsWith(pluginName)) {
                return this.preloads[p];
            }
        }
        return null;
    }
    runKeyShortcut(event, name, action) {
        const plugin = this.findPluginByName(name);
        if (plugin === null) {
            log_1.default.error(`While trying to execute key shortcut '${action}', plugin for '${name}' not found:`, this.preloads);
            return;
        }
        const f = (plugin.keymaps || {})[action];
        if (!f) {
            log_1.default.error(`There is no key shortcut action '${action}' in plugin '${name}'`, plugin);
            return;
        }
        try {
            f(event, this.config, this.account);
        }
        catch (e) {
            log_1.default.error(`Error while executing plugin-defined key short action: ${action} with plugin '${name}'`, e);
        }
    }
}
exports.default = PluginsLoader;
//# sourceMappingURL=plugins.js.map